use crate::ips::IpType;

use super::providers::Providers;
use serde::Deserialize;

#[derive(Deserialize)]
pub(crate) struct Config {
    /// Default configuration for domains.
    pub(crate) defaults: Option<Defaults>,
    /// How often to update dns records (measured in seconds).
    #[serde(default = "i_def")]
    pub(crate) interval: u64, // Type required by Duration::from_secs()
    /// A vector of all domains to be updated
    pub(crate) domains: Vec<Providers>,
    /// Whether to only update records if a change in ip address is detected. On the first run records will always be updated.
    #[serde(default = "uoic_def")]
    pub(crate) only_update_on_ip_change: bool,
}

/// Default configeration for domains, given they accept these fields and do not have an overwrite set.
#[derive(Deserialize)]
pub(crate) struct Defaults {
    #[serde(default = "ttl_def")]
    pub(crate) ttl: u32,
    #[serde(default = "it_def")]
    pub(crate) ip_type: IpType,
}

fn uoic_def() -> bool {
    true
}

fn i_def() -> u64 {
    600
}

fn it_def() -> IpType {
    IpType::V4
}

fn ttl_def() -> u32 {
    600
}
