use pnet::{datalink::interfaces, ipnetwork::IpNetwork};
use serde::Deserialize;

use std::net::{Ipv4Addr, Ipv6Addr};

/// Gets the public ip addresses of the computer, with the values in the tuple representing v4 and v6 addresses respectively.
pub(crate) fn get_ip() -> (Option<Ipv4Addr>, Option<Ipv6Addr>) {
    let all_interfaces = interfaces();

    let default_interface = all_interfaces
        .iter()
        .find(|x| x.is_up() && !x.is_loopback() && !x.ips.is_empty());

    match default_interface {
        Some(interface) => (
            interface
                .ips
                .iter()
                .filter_map(|x| match x {
                    IpNetwork::V6(_) => None,
                    IpNetwork::V4(net) => {
                        if !net.ip().is_private() {
                            Some(net.ip())
                        } else {
                            match igd::search_gateway(Default::default()) {
                                Err(ref err) => {
                                    println!("Error: {}", err);
                                    None
                                }
                                Ok(gateway) => match gateway.get_external_ip() {
                                    Err(ref _err) => None,
                                    Ok(ext_addr) => Some(ext_addr),
                                },
                            }
                        }
                    }
                })
                .find(|x| !x.is_loopback() && !x.is_unspecified()),
            interface
                .ips
                .iter()
                .filter_map(|x| match x {
                    IpNetwork::V6(y) => Some(y.ip()),
                    IpNetwork::V4(_) => None,
                })
                .find(|x| !x.is_loopback() && !x.is_unspecified()),
        ),
        None => (None, None),
    }
}

#[derive(Deserialize, PartialEq, Copy, Clone)]
pub(crate) enum IpType {
    Both,
    V4,
    V6,
}
